#ifndef _MATRIX_HEADER
#define _MATRIX_HEADER

#include <stdbool.h>
#include <stdint.h>


typedef int32_t T;


typedef struct matrix
{
    uint32_t row;
    uint32_t col;
    T **data;
} matrix;

#include "traitementPGM.h"


bool matrix_is_equal(const matrix *const mat1, const matrix *const mat2);
matrix *matrix_create(uint32_t col, uint32_t row);
void matrix_destroy(matrix *mat);
void matrix_print(const matrix *const mat);
void matrix_set_values(matrix *mat, T value);
matrix* matrix_create_from_array(uint32_t col, uint32_t row, T *data, uint32_t data_length);
void matrix_to_array(const matrix *const mat, T *table);
matrix *matrix_clone(const matrix *const mat);
bool _matrix_crash_test(const matrix *const mat);
void matrix_get(T *elem, const matrix *const mat, uint32_t id_x, uint32_t id_y);
void matrix_set(const matrix *mat, uint32_t id_x, uint32_t id_y, const T elem);
matrix *matrix_map(const matrix *const mat, T (*foo)(T *));
void matrix_map_ip(const matrix *mat, void (*func)(T *));
matrix* matrix_extract_submatrix( const matrix* const mat, uint32_t x0, uint32_t x1, uint32_t y0, uint32_t y1);
matrix* matrix_transpose(const matrix* const mat);
matrix *matrix_padding(const matrix* const mat, uint32_t number_of_padding);
// matrix *matrix_convolve(matrix * mat, const matrix *const kernel, const int32_t factor,const int32_t max_value);
matrix* matrix_create_random_matrix(uint32_t col,uint32_t row,uint32_t seed,uint32_t modulus);
void _print_data_table(uint32_t data_length,T* data);
void matrix_init(matrix* mat,uint32_t col,uint32_t row);
T matrix_min(matrix *orig);
T matrix_max(matrix* orig);
void matrix_free(matrix* mat);
// matrix* matrix_average_kernel(const matrix *const mat, const matrix *const kernel);
// void pgm_print(pgm* p);
// pgm* pgm_read_p5_file(const char* const filename);
// void pgm_destroy(pgm* p);
// void pgm_write_p5_to_file(pgm *p, char *filename);
// pgm* pgm_convolv(pgm* orig,matrix* kernel);

#endif