#include "matrix.h"

matrix matrix_creat(int m, int n)
{
    matrix tmp;
    tmp.pM = NULL;
    matrix_init(&tmp, m, n);
    return tmp;
}

error_code matrix_reset(matrix *mat)
{
    for (int i = 0; i < mat->n; i += 1)
    {
        for (int j = 0; j < mat->m; j += 1)
        {
            mat->pM[i][j] = 0;
        }
    }
    if (mat->pM == NULL)
    {
        return uninitialized;
    }
    return ok;
}
error_code matrix_print(const matrix mat)
{
    if (mat.pM != NULL)
    {
        printf("\n");
        for (int i = 0; i < mat.n; i++)
        {
            printf("%d ", i);
        }
        printf("'n' Colunm\n\n");

        for (int i = 0; i < mat.m; i++)
        {
            for (int j = 0; j < mat.n; j++)
            {
                printf("%d ", mat.pM[j][i]);
            }
            printf("   %d", i);
            printf("\n");
        }
        printf("           'm' ligne\n");
        printf("\n");
        return ok;
    }
    else
    {
        return uninitialized;
    }
}

error_code matrix_init(matrix *mat, int32_t m, int32_t n)
{
    if (m > 0 && n > 0)
    {
            mat->pM = calloc(n , sizeof(int32_t *)); // allouer l'espace pour un pointeur
            mat->n = n; // x
            mat->m = m; // y

            for (int i = 0; i < n; i++)
            {
                mat->pM[i] = calloc(m , sizeof(int32_t));
            }

            // for (int i = 0; i < n; i++)
            // {
            //     for (int j = 0; j < m; j++)
            //     {
            //         mat->pM[i][j] = 0;
            //         // printf("p[%u][%u] = %d\n", i, j, mat->pM[i][j]);
            //     }
            // }
        

        return ok;
    }
    else
    {
        return out_of_bounds;
    }
}

error_code matrix_init_from_array(matrix *mat, int32_t m, int32_t n, int32_t data[])
{
    if (m > 0 && n > 0)
    {
        if (mat->pM == NULL)
        {
            if (matrix_init(mat, m, n) == ok)
            {
                for (int i = 0; i < m; ++i)
                {
                    for (int j = 0; j < n; ++j)
                    {
                        mat->pM[j][i] = data[i * n + j];
                        // printf("p[%u][%u] = %d\n", i, j, mat->pM[i][j]);
                    }
                }
            }
        }
        else
        {
            return memory_error;
        }

        return ok;
    }
    else
    {
        return out_of_bounds;
    }
}

error_code matrix_from_array(matrix *mat, int32_t m, int32_t n, int32_t data[])
{
    if (m > 0 && n > 0)
    {
        if (mat->pM != NULL)
        {
            for (int i = 0; i < m; ++i)
            {
                for (int j = 0; j < n; ++j)
                {
                    mat->pM[j][i] = data[i * n + j];
                    // printf("p[%u][%u] = %d\n", i, j, mat->pM[i][j]);
                }
            }
        }
        else
        {
            return memory_error;
        }

        return ok;
    }
    else
    {
        return out_of_bounds;
    }
}

error_code matrix_clone(matrix *cloned, const matrix mat)
{
    if (mat.pM != NULL)
    {
        if (cloned->pM == NULL)
        {
            if (matrix_init(cloned, mat.m, mat.n) == ok)
            {
                for (int i = 0; i < mat.n; i++)
                {
                    for (int j = 0; j < mat.m; j++)
                    {
                        cloned->pM[i][j] = mat.pM[i][j];
                    }
                }
                return ok;
            }
            return uninitialized;
        }
        else
        {
            return memory_error;
        }
    }
    else
    {
        return uninitialized;
    }
}

error_code matrix_transpose(matrix *transposed, const matrix mat)
{
    if (mat.pM != NULL)
    {
        if (transposed->pM == NULL)
        {

            if (matrix_init(transposed, mat.n, mat.m) == ok)
            {
                for (int i = 0; i < transposed->n; i++)
                {
                    for (int j = 0; j < transposed->m; j++)
                    {
                        transposed->pM[i][j] = mat.pM[j][i];
                    }
                }
                return ok;
            }
            return uninitialized;
        }
        else
        {
            return memory_error;
        }
    }
    else
    {
        return uninitialized;
    }
}

error_code matrix_extract_submatrix(matrix *sub, const matrix mat, int32_t m0, int32_t m1, int32_t n0, int32_t n1)
{
    if (mat.pM != NULL)
    {
        if (sub->pM == NULL)
        {
            if (mat.m > m1 && mat.n > n1)
            {
                matrix_init(sub, m1 - m0, n1 - n0);

                for (int i = 0; i < n1 - n0; i++)
                {
                    for (int j = 0; j < m1 - m0; j++)
                    {
                        sub->pM[i][j] = mat.pM[i + n0][j + m0];
                    }
                }
            }
            else
            {
                return out_of_bounds;
            }

            return ok;
        }
        else
        {
            return memory_error;
        }
    }
    else
    {
        return uninitialized;
    }
}

bool matrix_is_equal(matrix mat1, matrix mat2)
{
    if (mat1.pM != NULL && mat2.pM != NULL)
    {
        if (mat1.m == mat2.m && mat1.n == mat2.n)
        {
            for (int i = 0; i < mat1.n; i++)
            {
                for (int j = 0; j < mat1.m; j++)
                {
                    if (mat1.pM[i][j] != mat2.pM[i][j])
                    {
                        return false;
                    }
                }
            }
            return true;
        }
    }
    return false;
}

error_code matrix_get(int32_t *elem, const matrix mat, int32_t ix, int32_t iy)
{
    if (mat.pM != NULL)
    {
        if (mat.m > ix && mat.n > iy)
        {
            *elem = mat.pM[iy][ix];
            printf("item pos m%d,n%d = %d\n", ix, iy, mat.pM[iy][ix]);
            return ok;
        }
        else
        {
            return out_of_bounds;
        }
    }
    else
    {
        return uninitialized;
    }
}

error_code matrix_set(matrix *mat, int32_t ix, int32_t iy, int32_t elem)
{
    if (mat->pM != NULL)
    {
        if (mat->m > ix && mat->n > iy)
        {
            mat->pM[iy][ix] = elem;
            printf("item pos m%d,n%d = %d\n", ix, iy, mat->pM[iy][ix]);
            return ok;
        }
        else
        {
            return out_of_bounds;
        }
    }
    else
    {
        return uninitialized;
    }
}

error_code matrix_map_ip(matrix mat, void (*foo)(int *))
{
    // foo(&mat.pM[1][1]); //test
    if (mat.pM != NULL)
    {
        for (int i = 0; i < mat.m; i++)
        {
            for (int j = 0; j < mat.n; j++)
            {
                foo(&mat.pM[j][i]);
            }
        }
        return ok;
    }
    else
    {
        return uninitialized;
    }
}
error_code matrix_map(matrix *mapped, const matrix mat, void (*foo)(int *))
{
    if (matrix_clone(mapped, mat) == ok)
    {
        matrix_map_ip(*mapped, foo);
        return ok;
    }
    else
    {
        return uninitialized;
    }
}
void mult2(int32_t *a)
{
    *a *= 2;
}

error_code matrix_destroy(matrix *mat)
{
    for (int i = 0; i < mat->n; ++i)
        free(mat->pM[i]);

    free(mat->pM);

    mat->pM = NULL;
    mat->m = -1;
    mat->n = -1;

    return ok;
}

error_code matrix_setValue(matrix *mat, int value)
{
    for (int i = 0; i < mat->m; i++)
    {
        for (int j = 0; j < mat->n; j++)
        {
            matrix_set(mat,i,j,value);
        }
        
    }

    return ok;
    
}